/*********************************************************************
 *
 *                Microchip USB Bootloader Version 1.2
 *
 *********************************************************************
 * FileName:        main.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 3.11+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 * Rawin Rojvanit       05/14/07    Minor updates.
 ********************************************************************/

/*********************************************************************
IMPORTANT NOTE: This booloader example code is not intended to work
with the PIC18F87J50 Family of microcontrollers.
*********************************************************************/


/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "system\typedefs.h"                        // Required
#include "system\usb\usb.h"                         // Required
#include "io_cfg.h"                                 // Required

#include "system\usb\usb_compile_time_validation.h" // Optional

/** C O N F I G U R A T I O N ************************************************/

#if   defined(__18F4550)||defined(__18F4455)|| \
      defined(__18F2550)||defined(__18F2455)|| \
      defined(__18F4553)||defined(__18F4458)|| \
      defined(__18F2553)||defined(__18F2458)

// Note: Some of the below configuration bits are commented out
// to prevent build errors with some of the above listed devices.
// For example, on the PIC18F4458 CP3, WRT3, and EBTR3 don't exist.

//#pragma config PLLDIV   = 5       // (20 MHz input)
//#pragma config PLLDIV   = 4       // 16 MHz input
//#pragma config PLLDIV   = 3       // 12 MHz input
#pragma config PLLDIV   = 2       //  8 MHz input
//#pragma config PLLDIV   = 1       //  4 MHz input
#pragma config CPUDIV   = OSC1_PLL2 //OSC1/OSC2 Src: /1, 96 MHz PLL Src: /2
#pragma config USBDIV   = 2       // Clock source from 96MHz-PLL/2
#pragma config FOSC     = HSPLL_HS  //HS oscillator, PLL enabled, HS used by USB
//#pragma config FCMEN    = OFF		//Fail-Safe Clock Monitor disabled 
#pragma config IESO     = OFF		//Oscillator Switchover mode disabled 
#pragma config PWRT     = ON		//PWRT enabled
#pragma config BOR      = OFF		//Brown-out Reset disabled in hardware and software
//#pragma config BORV     = 3
#pragma config VREGEN   = ON		//USB voltage regulator enabled
//StK??: VRegEn notwendig wenn self-powered?
#pragma config WDT      = OFF		//Watchdog: HW Disabled - SW Controlled		
#pragma config WDTPS    = 32768		//Watchdog Timer Postscale 1:32768
//#pragma config MCLRE    = OFF		//RE3 input pin enabled; MCLR disabled
//StK: MCLR wird als RESET verwendet
#pragma config LPT1OSC  = OFF		//Timer1 configured for higher power operation	
#pragma config PBADEN   = OFF		//PORTB<4:0> pins are configured as digital I/O on Reset
#pragma config CCP2MX   = ON		//CCP2 input/output is multiplexed with RC1
#pragma config STVREN   = ON		//Stack full/underflow will cause Reset 
#pragma config LVP      = OFF		//Single-Supply ICSP disabled
//#pragma config ICPRT    = OFF       // Dedicated In-Circuit Debug/Programming
#pragma config XINST    = OFF       //Extended Instruction Set Off
#pragma config CP0      = OFF		//Block 0 (000800-001FFFh) not code-protected
#pragma config CP1      = OFF		//Block 1 (002000-003FFFh) not code-protected
#pragma config CP2      = OFF		//Block 2 (004000-005FFFh) not code-protected
//#pragma config CP3      = OFF		//Block 3 (006000-007FFFh) not code-protected
#pragma config CPB      = OFF		//Boot block (000000-0007FFh) not code-protected
#pragma config CPD      = OFF		//Data EEPROM not code-protected
#pragma config WRT0     = OFF		//Block 0 (000800-001FFFh) not write-protected
#pragma config WRT1     = OFF		//Block 1 (002000-003FFFh) not write-protected
#pragma config WRT2     = OFF		//Block 2 (004000-005FFFh) not write-protected
//#pragma config WRT3     = OFF		//Block 3 (006000-007FFFh) not write-protected
#pragma config WRTB     = OFF       //Boot block (000000-0007FFh) not write-protected
#pragma config WRTC     = OFF		//Configuration registers (300000-3000FFh) not write-protected
#pragma config WRTD     = OFF		//Data EEPROM not write-protected 
#pragma config EBTR0    = OFF		//Block 0 not protected from table reads executed in other blocks
#pragma config EBTR1    = OFF		//Block 1 not protected from table reads executed in other blocks 
#pragma config EBTR2    = OFF		//Block 2 not protected from table reads executed in other blocks 
//#pragma config EBTR3    = OFF		//Block 3 not protected from table reads executed in other blocks 
#pragma config EBTRB    = OFF		//Boot block not protected from table reads executed in other blocks

#endif

/** V A R I A B L E S ********************************************************/
#pragma udata

/** P R I V A T E  P R O T O T Y P E S ***************************************/

/** V E C T O R  R E M A P P I N G *******************************************/

#pragma code _HIGH_INTERRUPT_VECTOR = 0x000008
void _high_ISR (void)
{
    _asm goto RM_HIGH_INTERRUPT_VECTOR _endasm
}

#pragma code _LOW_INTERRUPT_VECTOR = 0x000018
void _low_ISR (void)
{
    _asm goto RM_LOW_INTERRUPT_VECTOR _endasm
}

#pragma code

/** D E C L A R A T I O N S **************************************************/
#pragma code
/******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *****************************************************************************/
void main(void)
{
  // eine Brcke von RE3 (MCLR-Pin1) nach Masse kennzeichnet Bootloadermode
  // externer pull-up vorhanden
  // RE3==1  ->  Springe zur Firmware
  // RE3==0  ->  Bootloader
//StK: RA0 = Taster fr Bootloadermode
//   RA0==1  -> Firmware
//   RA0==0  -> Bootloader

    //Check Bootload Mode Entry Condition
	TRISA = 0b11111111;    
	if (PORTAbits.RA0 == 1)  // Taster nicht gedrckt
    {
        _asm goto RM_RESET_VECTOR _endasm
    }//end if
    
    //Bootload Mode
    mInitAllLEDs();
	mLED_1_On();				// LED an  
    mInitializeUSBDriver();     // See usbdrv.h
    USBCheckBusStatus();        // Modified to always enable USB module
    while(1)
    {
        USBDriverService();     // See usbdrv.c
        BootService();          // See boot.c
    }//end while
}//end main

#pragma code user = RM_RESET_VECTOR

/** EOF main.c ***************************************************************/
